import React from 'react';

import { Navigate, useLocation } from 'react-router-dom';
import PageNotFound from '../common/lib/PageNotFound';
import { MultiLang } from '../config';
import Functions from '../functions';

interface Props {
  lang: MultiLang;
}

const XoonipsXoopsPathRedirect: React.FC<Props> = (props) => {
  const { lang } = props;
  const location = useLocation();

  const getRedirectUrl = (): string => {
    const pathname = location.pathname || '';
    const query = new URLSearchParams(location.search);
    const search = new RegExp('^/modules/xoonips(?:/+(.*))?$');
    const matches = pathname.match(search);
    if (matches === null) {
      return '';
    }
    const path = matches[1] || '';
    switch (path) {
      case '':
      case 'index.php': {
        return '/database';
      }
      case 'detail.php': {
        const id = query.get('id');
        if (id !== null) {
          return '/database/item/id/' + Functions.escape(id);
        }
        const itemId = query.get('item_id');
        if (itemId?.match(/^\d+$/) != null) {
          return '/database/item/' + Functions.escape(itemId);
        }
        return '';
      }
      case 'listitem.php': {
        const indexId = query.get('index_id');
        if (indexId?.match(/^\d+$/) != null) {
          const params = new URLSearchParams();
          [
            { qKey: 'orderby', pKey: 'orderby', isNumber: false },
            { qKey: 'order_dir', pKey: 'order_dir', isNumber: true },
            { qKey: 'itemcount', pKey: 'itemcount', isNumber: true },
            { qKey: 'page', pKey: 'page', isNumber: true },
          ].forEach(({ qKey, pKey, isNumber }) => {
            const v = query.get(qKey);
            if (v == null || v.length === 0) {
              return;
            }
            if (isNumber && v.match(/^\d+$/) != null) {
              return;
            }
            params.set(pKey, v);
          });
          const paramStr = params.toString();
          return (
            `/database/list/${Functions.escape(indexId)}` +
            (paramStr.length > 0 ? `?${paramStr}` : '')
          );
        }
        break;
      }
      case 'itemselect.php': {
        const op = query.get('op');
        if (op === null) {
          break;
        }
        switch (op) {
          case 'quicksearch': {
            const keyword = query.get('keyword');
            const itemType = query.get('search_itemtype');
            if (keyword === null || itemType === null || keyword === '') {
              return '';
            }
            const type = itemType.replace('xnp', '');
            if (itemType !== 'basic' && itemType !== 'all' && itemType.match(/^xnp.+/) === null) {
              return '';
            }
            const params = new URLSearchParams({ type, keyword });
            [
              { qKey: 'orderby', pKey: 'orderby', isNumber: false },
              { qKey: 'orderdir', pKey: 'order_dir', isNumber: true },
              { qKey: 'item_per_page', pKey: 'itemcount', isNumber: true },
              { qKey: 'page', pKey: 'page', isNumber: true },
            ].forEach(({ qKey, pKey, isNumber }) => {
              const v = query.get(qKey);
              if (v == null || v.length === 0) {
                return;
              }
              if (isNumber && v.match(/^\d+$/) != null) {
                return;
              }
              params.set(pKey, v);
            });
            return '/database/search?' + params.toString();
          }
          case 'itemtypesearch': {
            const itemType = query.get('search_itemtype');
            if (itemType?.match(/^xnp.+/) == null) {
              return '';
            }
            const type = itemType.replace('xnp', '');
            return '/database/search/itemtype/' + Functions.escape(type);
          }
          case 'itemsubtypesearch': {
            let type = '';
            let subtype = '';
            query.forEach((v, k) => {
              if (k.match(/^xnp[a-z]+$/) !== null && !!v) {
                type = k.replace('xnp', '');
                return;
              }
            });
            if (type === '') {
              return '';
            }
            query.forEach((v, k) => {
              if (k.match(`^xnp${type}_.+$`) !== null && !!v) {
                subtype = v;
                return;
              }
            });
            if (subtype === '') {
              return '';
            }
            return (
              '/database/search/itemtype/' +
              Functions.escape(type) +
              '/' +
              Functions.escape(subtype)
            );
          }
        }
        return '';
      }
      case 'advanced_search.php': {
        return '/database/advanced';
      }
    }
    return '';
  };

  const url = getRedirectUrl();
  if (url === '') {
    return <PageNotFound lang={lang} />;
  }
  return <Navigate to={url} />;
};

export default XoonipsXoopsPathRedirect;
